@@ -1,8 +1,9 @@
-"drm" <-
+`drm` <-
   function (formula, family = binomial, data = sys.parent(), weights, 
             offset, subset = NULL, na.action, start = NULL, link = "cum", 
-            dep = "I", Ncond = TRUE, Lclass=2, dropout = FALSE, drop.x = NULL,
-            save.profiles = TRUE, pmatrix = NULL, print.level = 2, iterlim = 200, ...) 
+            dep = "I", Ncond = TRUE, Lclass = 2, dropout = FALSE, drop.x = NULL, 
+            save.profiles = TRUE, pmatrix = NULL, print.level = 2, iterlim = 200, 
+            ...) 
 {
   if (length(dep) > 1) {
     assoc <- dep[2:length(dep)]
@@ -50,16 +51,16 @@
   ord <- attr(Terms, "order")[c(tempc$terms, tempt$terms)]
   if (any(ord > 1)) 
     stop("cluster() or Time() can not be used in an interaction")
-  if (length(grep("M", dep)) > 0){
+  if (length(grep("M", dep)) > 0) {
     ord <- c(cluster, Time)
     note <- paste("Note:  ordering of the responses within each cluster is by Time")
   }
-  else{
-    if(any(is.na(m))){
+  else {
+    if (any(is.na(m))) {
       ord <- c(cluster, attr(Terms, "response"))
       note <- paste("Note:  ordering of the responses within each cluster is by response value (with NA last)")
     }
-    else{
+    else {
       ord <- cluster
       note <- NULL
     }
@@ -77,8 +78,7 @@
   dimnames(X)[[1]] <- sort(as.numeric(as.factor(cl)))
   nrep <- table(table(cluster))
   if (length(nrep) > 1) 
-    stop(paste("Can handle only balanced data frame.",
-               "For unbalanced data frames, fill NA's to obtain equal cluster size", 
+    stop(paste("Can handle only balanced data frame.", "For unbalanced data frames, fill NA's to obtain equal cluster size", 
                sep = "\n"))
   nrep <- as.numeric(names(nrep))
   wt <- model.extract(m, weights)
@@ -94,8 +94,10 @@
   if (!length(wt)) 
     wt <- 1
   offset <- model.extract(m, offset)
-  if (length(offset) <= 1) 
-    offset <- rep(0, n)
+  if (length(offset) == 1) 
+    offset <- rep(offset, n)
+  if (length(offset)==0)
+    offset <- rep(0,n)
   y <- model.extract(m, response)
   if (any(is.na(y))) 
     cat(paste("Note: Missing response values; Assuming", 
@@ -108,12 +110,14 @@
   mis <- apply(cbind(y, X), 1, function(i) any(is.na(i)))
   x <- X[!mis, drop = FALSE, ]
   y1 <- ifelse(as.numeric(as.factor(y)) > 1, 1, 0)[!mis]
-  if(is.null(start)){
+  if (is.null(start)) {
     parm1 <- glm.fit(x, y1, family = family)$coefficients
     if (nclass > 2) {
-      theta <- matrix(0, nrow = length(parm1), ncol = nclass - 2)
+      theta <- matrix(0, nrow = length(parm1), ncol = nclass - 
+                      2)
       for (i in 2:(nclass - 1)) {
-        y1 <- ifelse(as.numeric(as.factor(y)) > i, 1, 0)[!mis]
+        y1 <- ifelse(as.numeric(as.factor(y)) > i, 1, 
+                     0)[!mis]
         theta[, i - 1] <- glm.fit(x, y1, family = family)$coefficients
       }
       parms <- c(as.vector(-parm1[1]), (-theta[1, ]), parm1[-1])
@@ -123,11 +127,12 @@
         parms[1:(nclass - 1)] <- log(table(y)[nclass]) - 
           log(table(y)[-nclass])
         bclreg <- as.vector(-theta[-1, ])
-        names(bclreg) <- rep(names(parm1)[-1], nclass - 2)
+        names(bclreg) <- rep(names(parm1)[-1], nclass - 
+                             2)
         parms <- c(-parms, bclreg)
-        names(parms)[-(1:(nclass - 1))] <-
-          paste(names(parms)[-(1:(nclass -  1))],
-                rep(1:(nclass - 1), rep(nrow(theta) - 1, nclass - 1)), sep = "")
+        names(parms)[-(1:(nclass - 1))] <- paste(names(parms)[-(1:(nclass - 
+                                                                   1))], rep(1:(nclass - 1), rep(nrow(theta) - 
+                                                                                                 1, nclass - 1)), sep = "")
       }
       if (link == "acl") {
         parms[1:(nclass - 1)] <- log(table(y)[-nclass]) - 
@@ -136,12 +141,14 @@
     }
     else (parms <- parm1)
   }
-  else{
-    parms <- rep(0,ncol(x)+(nclass-2))
+  else {
+    parms <- rep(0, ncol(x) + (nclass - 2))
     names(parms)[1:(nclass - 1)] <- paste("(Intercept)", 
-                                          if(nclass>2) 1:(nclass - 1) else(""), sep = "")
-    names(parms)[nclass:(ncol(x)+nclass-2)] <- dimnames(x)[[2]][-1]
-   }
+                                          if (nclass > 2) 
+                                          1:(nclass - 1)
+                                          else (""), sep = "")
+    names(parms)[nclass:(ncol(x) + nclass - 2)] <- dimnames(x)[[2]][-1]
+  }
   npar <- length(parms)
   if (length(grep("N", dep)) > 0) 
     parms <- c(parms, nu = 1)
@@ -163,22 +170,24 @@
     }
   }
   if (length(grep("L", dep)) > 0) {
-    if(Lclass>2){
-      if(nclass>2)
+    if (Lclass > 2) {
+      if (nclass > 2) 
         stop("Lclass > 2 for structure L currently implemented only for binary responses")
-      if(length(grep("LM", dep)) > 0)
-        stop("Lclass > 2 for structure LM currently not implemented")        
-     else{
-       p2 <- c(rep(0.01/(Lclass-2),Lclass-2),0.99, c(1:(Lclass-1))/Lclass)
-       names(p2) <- c(paste("nu",1:(Lclass - 1),sep = "") ,
-                      paste("kappa", 0:(Lclass - 2),sep = ""))
+      if (length(grep("LM", dep)) > 0) 
+        stop("Lclass > 2 for structure LM currently not implemented")
+      else {
+        p2 <- c(rep(0.01/(Lclass - 2), Lclass - 2), 0.99, 
+                c(1:(Lclass - 1))/Lclass)
+        names(p2) <- c(paste("nu", 1:(Lclass - 1), sep = ""), 
+                       paste("kappa", 0:(Lclass - 2), sep = ""))
       }
     }
-    else{
+    else {
       p2 <- c(nu1 = 0.99, rep(0.5, nclass - 1))
-      if(nclass==2)
+      if (nclass == 2) 
         names(p2) <- c("nu1", "kappa")
-      else(names(p2) <- c("nu1", paste("kappa", 1:(nclass - 1), sep = "")))
+      else (names(p2) <- c("nu1", paste("kappa", 1:(nclass - 
+                                                    1), sep = "")))
     }
     parms <- c(parms, p2)
   }
@@ -208,9 +217,9 @@
       names(dropx) <- c(paste(names(m)[1], c("cur", "prev"), 
                               sep = "."), paste(deparse(substitute(drop.x)), 
                                 "prev", sep = "."))
-      misn <- lapply(seq(nrow(Y)), function(i, Y, dropc)
-                     c(Y[i, which(is.na(Y[i, ]) == TRUE)[1] - 1],
-                       dropc[i, which(is.na(Y[i, ]) == TRUE)[1] - 1]), Y = Y, 
+      misn <- lapply(seq(nrow(Y)), function(i, Y, dropc) c(Y[i, 
+                                                             which(is.na(Y[i, ]) == TRUE)[1] - 1], dropc[i, 
+                                                                          which(is.na(Y[i, ]) == TRUE)[1] - 1]), Y = Y, 
                      dropc = dropc)
     }
     else {
@@ -220,8 +229,8 @@
                        1, paste, collapse = "")
       names(dropx) <- c(paste(names(m)[1], c("cur", "prev"), 
                               sep = "."))
-      misn <- lapply(seq(nrow(Y)), function(i, Y, dropc)
-                     c(Y[i, which(is.na(Y[i, ]) == TRUE)[1] - 1]), Y = Y, 
+      misn <- lapply(seq(nrow(Y)), function(i, Y, dropc) c(Y[i, 
+                                                             which(is.na(Y[i, ]) == TRUE)[1] - 1]), Y = Y, 
                      dropc = dropc)
     }
     dropx <- eval(parse(text = paste("model.matrix(~", paste(names(dropx), 
@@ -270,7 +279,7 @@
   ass <- assoc$tfunction
   asscov <- assoc$ass
   constant <- assoc$constant
-  if (length(grep("M2", dep)) > 0 ) {
+  if (length(grep("M2", dep)) > 0) {
     if (nclass > 2) 
       stop("structure M2 currently implemented only for binary response")
   }
@@ -292,11 +301,8 @@
     stop(paste("Coefficient-names conflict with default association parameter names.\n", 
                "Provide different variable names for the regression coefficients"))
   f <- rep(list(1:nclass), if (length(grep("M", dep)) > 0) {
-    if (length(grep("M2", dep)) > 0) 
-      3
-    else 2
-  }
-  else nrep)
+    if (length(grep("M2", dep)) > 0) 3 else 2
+  } else nrep)
   mm <- expand.grid(f)
   tm <- sapply(1:(nclass - 1), function(i, mm) {
     apply(mm, 1, function(k, i) {
@@ -305,9 +311,8 @@
   }, mm = mm)
   if (length(asscov$assmodel) == 0) 
     asscov$alab <- rep(1, n/nrep)
-  Xlab <- apply(matrix(apply(cbind(X,rep(asscov$alab, rep(nrep, n/nrep))),
-                             1, paste, collapse = ""), ncol = nrep, byrow = TRUE), 
-                1, paste, collapse = "")
+  Xlab <- apply(matrix(apply(cbind(offset, X, rep(asscov$alab, rep(nrep, n/nrep))), 1, paste, collapse = ""),
+                       ncol = nrep, byrow = TRUE), 1, paste, collapse = "")
   lab <- match(dimnames(X)[[1]], grep("FALSE", paste(duplicated(Xlab))))
   X.org <- X
   offset.org <- offset
@@ -328,24 +333,25 @@
     .loglik <- get("loglikd.drm")
   }
   else (.loglik <- get("logliks.drm"))
-  tim <- system.time(llinit <- .loglik(parms, y = Y, X = X, dep = dep, 
-                                     asscov = asscov, tlen = tlen, equal = equal, npar = npar, 
-                                     nclass = nclass, nrep = nrep, tm = tm, w = w, lab = lab, 
-                                     inv = family$linkinv, ass = ass, link = link, constant = constant, 
-                                     Ncond = Ncond, Lclass=Lclass, offset = offset, wt = wt, Nf = Nf, misn = misn, 
-                                     drop.cov = dropc, dropx = dropx, droplab = droplab))
+  tim <- system.time(llinit <- .loglik(parms, y = Y, X = X, 
+                                       dep = dep, asscov = asscov, tlen = tlen, equal = equal, 
+                                       npar = npar, nclass = nclass, nrep = nrep, tm = tm, w = w, 
+                                       lab = lab, inv = family$linkinv, ass = ass, link = link, 
+                                       constant = constant, Ncond = Ncond, Lclass = Lclass, 
+                                       offset = offset, wt = wt, Nf = Nf, misn = misn, drop.cov = dropc, 
+                                       dropx = dropx, droplab = droplab))
   if (llinit == .Machine$double.xmax) 
     stop("Poor initial regression parameter values: set new with `start'-argument")
   cat(paste("Initial log likelihood", -round(llinit, 2), ";", 
             "Function evaluation time:", round(tim[3], 2), "sec.\n"))
   cat("Starting numerical optimisation of log likelihood...\n\n")
-  est <- nlm(f = .loglik, p = parms, hessian = TRUE, y = Y, X = X, 
-             dep = dep, asscov = asscov, npar = npar, nclass = nclass, 
+  est <- nlm(f = .loglik, p = parms, hessian = TRUE, y = Y, 
+             X = X, dep = dep, asscov = asscov, npar = npar, nclass = nclass, 
              nrep = nrep, tm = tm, w = w, print.level = print.level, 
              lab = lab, equal = equal, inv = family$linkinv, ass = ass, 
-             link = link, constant = constant, Ncond = Ncond, Lclass=Lclass, tlen = tlen, 
-             offset = offset, wt = wt, Nf = Nf, misn = misn, dropx = dropx, 
-             droplab = droplab, drop.cov = dropc, iterlim = iterlim, 
+             link = link, constant = constant, Ncond = Ncond, Lclass = Lclass, 
+             tlen = tlen, offset = offset, wt = wt, Nf = Nf, misn = misn, 
+             dropx = dropx, droplab = droplab, drop.cov = dropc, iterlim = iterlim, 
              ...)
   estimate <- est$estimate
   names(estimate) <- names(parms)
@@ -371,7 +377,7 @@
                         tlen = tlen, npar = npar, nclass = nclass, equal = equal, 
                         nrep = nrep, tm = tm, w = w, lab = lab, inv = family$linkinv, 
                         ass = ass, link = link, constant = constant, Ncond = Ncond, 
-                        Lclass=Lclass,offset = offset, wt = wt, save.profiles = save.profiles, 
+                        Lclass = Lclass, offset = offset, wt = wt, save.profiles = save.profiles, 
                         asscov = asscov)
     cat("\nDone.\n")
     if (save.profiles) {
@@ -379,7 +385,7 @@
         v <- expand.grid(rep(list(1:nclass), nrep))
       dimnames(prof) <- list(apply(v, 1, paste, collapse = ""), 
                              unique(marg.names[, 1]))
-      if(!is.null(note))
+      if (!is.null(note)) 
         attributes(prof)$note <- note
     }
     if (nclass == 2) 
@@ -390,13 +396,13 @@
         if (npar == (nclass - 1)) 
           e.eta <- exp(sapply(estimate[1:(nclass - 1)], 
                               function(i, offs) i + offs, offs = offset.org))
-        else (e.eta <- exp(offset.org +
-                           sapply(0:(nclass -  2),
-                                  function(i, x, p, nclass) {
-                                    x %*% p[c(1 + i, (ncol(x) - 1) * i + (nclass:(ncol(x) + 
-                                                                                  nclass - 2)))]
-                                  }, x = X.org, p = estimate, nclass = nclass)))
-        mu <- e.eta/(1 + c(e.eta %*% rep(1, nclass -   1)))
+        else (e.eta <- exp(offset.org + sapply(0:(nclass - 
+                                                  2), function(i, x, p, nclass) {
+                                                    x %*% p[c(1 + i, (ncol(x) - 1) * i + (nclass:(ncol(x) + 
+                                                                                                  nclass - 2)))]
+                                                  }, x = X.org, p = estimate, nclass = nclass)))
+        mu <- e.eta/(1 + c(e.eta %*% rep(1, nclass - 
+                                         1)))
       }
       else {
         eta <- offset.org
@@ -404,8 +410,8 @@
           eta <- offset.org + (X.org[, -1, drop = FALSE] %*% 
                                estimate[nclass:npar])
         if (link == "cum") {
-          mu <- matrix(family$linkinv(sapply(estimate[1:(nclass -  1)],
-                                             function(i, eta) i - eta, eta = eta)), 
+          mu <- matrix(family$linkinv(sapply(estimate[1:(nclass - 
+                                                         1)], function(i, eta) i - eta, eta = eta)), 
                        ncol = (nclass - 1))
         }
         if (link == "acl") {
@@ -421,7 +427,7 @@
     dimnames(mu) <- list(apply(marg.names, 1, paste, collapse = ";"), 
                          names(estimate)[1:(nclass - 1)])
     mu <- mu[match(seq(nrow(mu)), mord), ]
-    if (length(grep("L", dep)) > 0){
+    if (length(grep("L", dep)) > 0) {
       if (nclass == 2) 
         mucond <- mu
       else {
@@ -441,14 +447,15 @@
         }
         else (assp <- assp[equal])
       }
-      if (is.na(match("nu1", names(estimate))) | Lclass > 2){
-        cat("Note: conditional probabilities not calculated; See ?getass.drm for an example\n") 
+      if (is.na(match("nu1", names(estimate))) | Lclass > 
+          2) {
+        cat("Note: conditional probabilities not calculated; See ?getass.drm for an example\n")
         mucond <- NULL
       }
       else {
-        L.p <- assp[match("nu1", names(estimate)
-                          [(npar +  1):length(estimate)]):
-                    (match("nu1", names(estimate)[(npar +  1):length(estimate)]) + (nclass - 1))]
+        L.p <- assp[match("nu1", names(estimate)[(npar + 
+                                                  1):length(estimate)]):(match("nu1", names(estimate)[(npar + 
+                                                                                                       1):length(estimate)]) + (nclass - 1))]
         mucond <- (t(t(mucond)/(L.p[1] + (1 - L.p[1]) * 
                                 L.p[-1])))
         mucond2 <- (t(t(mucond) * L.p[-1]))
@@ -474,4 +481,3 @@
                    terms = Terms), class = "drm")
   }
 }
-
